# -*- coding: utf-8 -*-

from django.http import HttpResponse
from django.shortcuts import render

from ..models import Publication


def by_unapi(request):
    """
    This view implements unAPI 1.0 (see http://unapi.info).
    """

    id = request.GET.get('id')
    format = request.GET.get('format')

    if format is not None:
        try:
            publications = Publication.objects.filter(pk=int(id))

            if not publications:
                raise ValueError

        except ValueError:
            # invalid id
            return HttpResponse('\n'.join(['<?xml version="1.0" encoding="UTF-8"?>', '<error>Invalid ID.</error>']),
                                content_type="application/xml", status=404)

        if format == 'bibtex':
            # return BibTex encoded publication
            return render(request, 'publications_bootstrap/export/publication.bib', {'publication': publications[0]},
                          content_type='text/x-bibtex; charset=UTF-8')

        if format == 'mods':
            # return MODS encoded publication
            return render(request, 'publications_bootstrap/export/publications.mods', {'publications': publications},
                          content_type='application/xml; charset=UTF-8')

        if format == 'ris':
            # return MODS encoded publication
            return render(request, 'publications_bootstrap/export/publications.ris', {'publications': publications},
                          content_type='application/x-research-info-systems; charset=UTF-8')

        # invalid format
        return HttpResponse('\n'.join(['<?xml version="1.0" encoding="UTF-8"?>', '<error>Invalid format.</error>']),
                            content_type="application/xml", status=406)

    if id is not None:
        return HttpResponse('\n'.join(['<?xml version="1.0" encoding="UTF-8"?>',
                                       '<formats id="{0}">'.format(id),
                                       '<format name="bibtex" type="text/x-bibtex" />',
                                       '<format name="ris" type="application/x-research-info-systems" />',
                                       '<format name="mods" type="application/xml" />',
                                       '</formats>']),
                            content_type="application/xml")

    return HttpResponse('\n'.join(['<?xml version="1.0" encoding="UTF-8"?>',
                                   '<formats>',
                                   '<format name="bibtex" type="text/x-bibtex" />',
                                   '<format name="ris" type="application/x-research-info-systems" />',
                                   '<format name="mods" type="application/xml" />',
                                   '</formats>']),
                        content_type="application/xml")



# -*- coding: utf-8 -*-

from django.db import models

from ..models import Publication


class PublicationLink(models.Model):
    class Meta:
        app_label = 'publications_bootstrap'  # Fix for Django<1.7

    publication = models.ForeignKey(Publication)
    description = models.CharField(max_length=256)
    url = models.URLField(verbose_name='URL')

    def __unicode__(self):
        return self.description

    def __str__(self):
        return self.description



# -*- coding: utf-8 -*-

from collections import defaultdict
from string import capwords

from django.shortcuts import render

from ..models import Type, Publication
from ..utils import populate


def by_author(request, name):
    fullname = capwords(name.replace('+', ' '))
    fullname = fullname.replace(' Von ', ' von ').replace(' Van ', ' van ')
    fullname = fullname.replace(' Der ', ' der ')

    # take care of dashes
    off = fullname.find('-')
    while off > 0:
        off += 1
        if off <= len(fullname):
            fullname = fullname[:off] + fullname[off].upper() + fullname[off + 1:]
        off = fullname.find('-', off)

    # split into forename, middlenames and surname
    names = name.replace(' ', '+').split('+')

    # construct a liberal query
    surname = names[-1]
    surname = surname.replace(u'ä', u'%%')
    surname = surname.replace(u'ae', u'%%')
    surname = surname.replace(u'ö', u'%%')
    surname = surname.replace(u'oe', u'%%')
    surname = surname.replace(u'ü', u'%%')
    surname = surname.replace(u'ue', u'%%')
    surname = surname.replace(u'ß', u'%%')
    surname = surname.replace(u'ss', u'%%')

    query_str = u'SELECT * FROM {table} ' \
                'WHERE lower({table}.authors) LIKE lower(\'%%{surname}%%\') ' \
                'ORDER BY {table}.year DESC, {table}.month DESC, {table}.id DESC'
    query = Publication.objects.raw(query_str.format(table=Publication._meta.db_table, surname=surname))

    # find publications of this author
    publications = []
    publications_by_type = defaultdict(lambda: [])

    # further filter results
    if len(names) > 1:
        name_simple = Publication.simplify_name(names[0][0] + '. ' + names[-1])
        for publication in query:
            if name_simple in publication.authors_list_simple:
                publications.append(publication)
                publications_by_type[publication.type_id].append(publication)

    elif len(names) > 0:
        for publication in query:
            if Publication.simplify_name(names[-1].lower()) in publication.authors_list_simple:
                publications.append(publication)
                publications_by_type[publication.type_id].append(publication)

    # attach publications to types
    types = Type.objects.filter(id__in=publications_by_type.keys())
    for t in types:
        t.publications = publications_by_type[t.id]

    if 'plain' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.txt', {'publications': publications},
                      content_type='text/plain; charset=UTF-8')

    if 'bibtex' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.bib', {'publications': publications},
                      content_type='text/x-bibtex; charset=UTF-8')

    if 'mods' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.mods', {'publications': publications},
                      content_type='application/xml; charset=UTF-8')

    if 'ris' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.ris', {'publications': publications},
                      content_type='application/x-research-info-systems; charset=UTF-8')

    if 'rss' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.rss',
                      {'url': 'http://' + request.get_host() + request.path,
                       'author': fullname,
                       'publications': publications
                       }, content_type='application/rss+xml; charset=UTF-8')

    # load custom links and files
    populate(publications)

    return render(request, 'publications_bootstrap/pages/author.html', {'publications': publications,
                                                                        'types': types,
                                                                        'author': fullname})



# -*- coding: utf-8 -*-

from django.shortcuts import render

from ..models import Publication
from ..utils import populate


def by_year(request, year=None):
    years = []
    publications = Publication.objects.select_related()
    if year:
        publications = publications.filter(year=year, external=False)
    else:
        publications = publications.filter(external=False)
    publications = publications.order_by('-year', '-month', '-id')

    for publication in publications:
        if publication.type.hidden:
            continue
        if not years or (years[-1][0] != publication.year):
            years.append((publication.year, []))
        years[-1][1].append(publication)

    if 'plain' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.txt',
                      {'publications': sum([y[1] for y in years], [])},
                      content_type='text/plain; charset=UTF-8')

    if 'bibtex' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.bib',
                      {'publications': sum([y[1] for y in years], [])},
                      content_type='text/x-bibtex; charset=UTF-8')

    if 'mods' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.mods',
                      {'publications': sum([y[1] for y in years], [])},
                      content_type='application/xml; charset=UTF-8')

    if 'ris' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.ris',
                      {'publications': sum([y[1] for y in years], [])},
                      content_type='application/x-research-info-systems; charset=UTF-8')

    if 'rss' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.rss',
                      {'url': 'http://' + request.get_host() + request.path,
                       'publications': sum([y[1] for y in years], [])},
                      content_type='application/rss+xml; charset=UTF-8')

    # load custom links and files
    populate(publications)

    return render(request, 'publications_bootstrap/pages/years.html', {'publications': publications,
                                                                       'years': years})



# -*- coding: utf-8 -*-

from django.shortcuts import render

from ..models import Catalog
from ..utils import populate


def for_catalog(request, title):
    try:
        catalog = Catalog.objects.get(title__iexact=title)

        publications = catalog.publication_set.all()
        publications = publications.order_by('-year', '-month', '-id')

        if 'plain' in request.GET:
            return render(request, 'publications_bootstrap/export/publications.txt', {'publications': publications},
                          content_type='text/plain; charset=UTF-8')

        if 'bibtex' in request.GET:
            return render(request, 'publications_bootstrap/export/publications.bib', {'publications': publications},
                          content_type='text/x-bibtex; charset=UTF-8')

        if 'mods' in request.GET:
            return render(request, 'publications_bootstrap/export/publications.mods', {'publications': publications},
                          content_type='application/xml; charset=UTF-8')

        if 'ris' in request.GET:
            return render(request, 'publications_bootstrap/export/publications.ris', {'publications': publications},
                          content_type='application/x-research-info-systems; charset=UTF-8')

        if 'rss' in request.GET:
            return render(request, 'publications_bootstrap/export/publications.rss', {
                'url': 'http://' + request.get_host() + request.path,
                'publications': publications
            }, content_type='application/rss+xml; charset=UTF-8')

        # load custom links and files
        populate(publications)

        return render(request, 'publications_bootstrap/pages/catalog.html', {
            'publications': publications,
            'title': "publications for catalog {}".format(title)})

    except Catalog.DoesNotExist:
        return render(request, 'publications_bootstrap/base.html', {
            'error': True,
            'alert': {'message': "There is no publication catalog with this name: {}".format(title)}},
                      status=404)



# -*- coding: utf-8 -*-

from django.db import models


class Catalog(models.Model):
    """
    Model representing a list of publications.
    """

    class Meta:
        ordering = ('title',)
        app_label = 'publications_bootstrap'  # Fix for Django<1.7

    title = models.CharField(max_length=128, unique=True)
    description = models.CharField(max_length=128)

    def __unicode__(self):
        return self.title

    def __str__(self):
        return self.title



# -*- coding: utf-8 -*-

from distutils.version import StrictVersion
from re import sub

import django
from django.template import Library, RequestContext
from django.template.loader import get_template
from django.utils.html import escape
from django.utils.safestring import mark_safe

from ..models import Publication, Catalog, Type
from ..utils import populate

register = Library()

GREEK_LETTERS = \
    '[Aa]lpha|[Bb]eta|[Gg]amma|[Dd]elta|[Ee]psilon|[Zz]eta|' + \
    '[Ee]ta|[Tt]heta|[Ll]ambda|[Mm]u|[Nn]u|[Pp]i|[Ss]igma|[Tt]au|' + \
    '[Pp]hi|[Pp]si|[Cc]hi|[Oo]mega|[Rr]ho|[Xx]i|[Kk]appa'


def render_template(template, request, args):
    if StrictVersion(django.get_version()) < StrictVersion('1.8.0'):
        return get_template(template).render(RequestContext(request, args))
    return get_template(template).render(args, request)


@register.simple_tag(takes_context=True)
def get_publications(context, template='publications_bootstrap/components/publications.html'):
    """
    Get all publications.
    """

    types = Type.objects.filter(hidden=False)
    publications = Publication.objects.select_related()
    publications = publications.filter(external=False, type__in=types)
    publications = publications.order_by('-year', '-month', '-id')

    if not publications:
        return render_template('publications_bootstrap/components/empty.html', context['request'], {})

    # load custom links and files
    populate(publications)

    return render_template(template, context['request'], {'publications': publications})


@register.simple_tag(takes_context=True)
def get_publication(context, p_id):
    """
    Get a single publication.
    """
    try:
        pbl = Publication.objects.get(pk=int(p_id))

        pbl.links = pbl.publicationlink_set.all()
        pbl.files = pbl.publicationfile_set.all()

        return render_template('publications_bootstrap/components/publication.html', context['request'],
                               {'publication': pbl})
    except Publication.DoesNotExist:
        return render_template('publications_bootstrap/components/empty.html', context['request'], {})


@register.simple_tag(takes_context=True)
def get_catalog(context, catalog_title, template='publications_bootstrap/components/section.html'):
    """
    Get a publication catalog.
    """
    try:
        publications_catalog = Catalog.objects.get(title__iexact=catalog_title)

        publications = publications_catalog.publication_set.all()
        if not publications:
            raise Publication.DoesNotExist
        publications = publications.order_by('-year', '-month', '-id')

        # load custom links and files
        populate(publications)

        return render_template(template, context['request'], {'title': catalog_title, 'publications': publications})
    except Catalog.DoesNotExist:
        return render_template('publications_bootstrap/components/empty.html', context['request'],
                               {'error': True, 'alert':
                                   {'heading': 'Zut!',
                                    'message': 'There is no such catalog named "%"'.format(catalog_title)}})
    except Publication.DoesNotExist:
        return render_template('publications_bootstrap/components/empty.html', context['request'], {})


@register.filter()
def tex_parse(string):
    """
    Renders some basic TeX math to HTML.
    """

    string = string.replace('{', '').replace('}', '')

    def tex_replace(match):
        return \
            sub(r'\^(\w)', r'<sup>\1</sup>',
                sub(r'\^\{(.*?)\}', r'<sup>\1</sup>',
                    sub(r'\_(\w)', r'<sub>\1</sub>',
                        sub(r'\_\{(.*?)\}', r'<sub>\1</sub>',
                            sub(r'\\(' + GREEK_LETTERS + ')', r'&\1;', match.group(1))))))

    return mark_safe(sub(r'\$([^\$]*)\$', tex_replace, escape(string)))


@register.filter(is_safe=False)
def as_list(o):
    return [o]



# -*- coding: utf-8 -*-

from django.db import models

from ordered_model.models import OrderedModel


class Type(OrderedModel):
    class Meta:
        ordering = ('order',)
        app_label = 'publications_bootstrap'  # Fix for Django<1.7

    title = models.CharField(max_length=128, unique=True)
    description = models.CharField(max_length=128)
    bibtex_types = models.CharField(max_length=256, default='article', verbose_name='BibTex types',
                                    help_text='Possible BibTex types, separated by comma.')
    hidden = models.BooleanField(default=False, help_text='Hide publications from main view.')

    def __unicode__(self):
        return self.title

    def __str__(self):
        return self.title

    def __init__(self, *args, **kwargs):
        OrderedModel.__init__(self, *args, **kwargs)

        self.bibtex_types = self.bibtex_types.replace('@', '')
        self.bibtex_types = self.bibtex_types.replace(';', ',')
        self.bibtex_types = self.bibtex_types.replace('and', ',')
        self.bibtex_type_list = [s.strip().lower() for s in self.bibtex_types.split(',')]
        self.bibtex_types = ', '.join(self.bibtex_type_list)
        self.bibtex_type = self.bibtex_type_list[0]

    def ris_type(self):
        # convert bibtex type to RIS type
        bibtex2ris = {
            'article': 'JOUR',
            'book': 'BOOK',
            'booklet': 'PAMP',
            'inbook': 'CHAP',
            'conference': 'CHAP',
            'inproceedings': 'CHAP',
            'incollection': 'CHAP',
            'manual': 'BOOK',
            'masterthesis': 'THES',
            'phdthesis': 'THES',
            'misc': 'GEN',
            'proceedings': 'CONF',
            'techreport': 'RPRT',
            'unpublished': 'UNPB',
            'patent': 'PAT',
            'abstract': 'ABST',
        }
        return bibtex2ris.get(self.bibtex_type, 'GEN')

    def mods_genre(self):
        """
        Guesses an appropriate MODS XML genre type.
        """

        type2genre = {
            'conference': 'conference publication',
            'book chapter': 'bibliography',
            'unpublished': 'article'
        }
        tp = str(self.title).lower()
        return type2genre.get(tp, tp)



# -*- coding: utf-8 -*-

from django import forms
from django.db import models
from django.forms import widgets


class PagesWidget(widgets.MultiWidget):
    def __init__(self, *args, **kwargs):
        attrs = {'style': 'width: 40px; text-align: center;'}
        forms.widgets.MultiWidget.__init__(self, [widgets.TextInput(attrs), widgets.TextInput(attrs)], *args, **kwargs)

    def format_output(self, rendered_widgets):
        to = ' <span style="vertical-align: middle;">to</span> '
        return rendered_widgets[0] + to + rendered_widgets[1]

    def decompress(self, value):
        if value:
            values = value.split('-')
            if len(values) > 1:
                return values
            if len(values) > 0:
                return [values[0], values[0]]
        return [None, None]


class PagesForm(forms.MultiValueField):
    widget = PagesWidget

    def __init__(self, *args, **kwargs):
        forms.MultiValueField.__init__(self, [forms.CharField(), forms.CharField()], *args, **kwargs)

    def compress(self, data_list):
        if data_list:
            if data_list[0] and data_list[1]:
                if data_list[0] == data_list[1]:
                    return str(data_list[0])
                return str(data_list[0]) + '-' + str(data_list[1])
            if data_list[0]:
                return str(data_list[0])
            if data_list[1]:
                return str(data_list[1])
        return ''


class PagesField(models.Field):
    def formfield(self, **kwargs):
        kwargs['form_class'] = PagesForm
        return models.Field.formfield(self, **kwargs)

    def get_internal_type(self):
        return 'CharField'


try:
    from south.modelsinspector import add_introspection_rules

    add_introspection_rules([], ["^publications_bootstrap\.fields\.PagesField"])
except:
    pass


class NullCharField(models.CharField):
    """
    Allows to apply the unique condition even if no value is set.
    Adapted from U{http://stackoverflow.com/a/33525694}
    U{Issue #4136<https://code.djangoproject.com/ticket/4136>} fixed in U{PR #6624<
    https://github.com/django/django/pull/6624>}
    Fixed in U{Django 1.11<https://docs.djangoproject.com/en/dev/releases/1.11/#miscellaneous>}
    """
    description = "CharField that stores NULL instead of empty strings"

    def get_db_prep_value(self, value, connection=None, prepared=False):
        value = super(NullCharField, self).get_db_prep_value(value, connection, prepared)
        if value == "":
            return None
        else:
            return value



# -*- coding: utf-8 -*-

from django.shortcuts import render

from ..models import Publication
from ..utils import populate


def by_tag(request, tag):
    tag = tag.lower().replace(' ', '+')
    candidates = Publication.objects.filter(tags__icontains=tag.split('+')[0], external=False)
    publications = []

    for i, publication in enumerate(candidates):
        if tag in [k[1] for k in publication.tags_escaped()]:
            publications.append(publication)

    if 'plain' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.txt', {'publications': publications},
                      content_type='text/plain; charset=UTF-8')

    if 'bibtex' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.bib', {'publications': publications},
                      content_type='text/x-bibtex; charset=UTF-8')

    if 'mods' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.mods', {'publications': publications},
                      content_type='application/xml; charset=UTF-8')

    if 'ris' in request.GET:
        return render(request, 'publications_bootstrap/export/publications.ris', {'publications': publications},
                      content_type='application/x-research-info-systems; charset=UTF-8')

    # load custom links and files
    populate(publications)

    return render(request, 'publications_bootstrap/pages/tag.html', {
        'publications': publications,
        'tag': tag.replace('+', ' '),
        'title': "publications for tag {}".format(tag.replace('+', ' '))})



# -*- coding: utf-8 -*-

from string import ascii_uppercase

from django.conf import settings
from django.db import models
from django.utils.http import urlquote_plus
from django.utils.translation import ugettext_lazy as _
from django_countries.fields import CountryField
from echoices.enums import EChoice, EOrderedChoice
from echoices.fields import make_echoicefield

from ..fields import NullCharField, PagesField
from ..models import Type, Catalog

if 'django.contrib.sites' in settings.INSTALLED_APPS:
    from django.contrib.sites.models import Site


class Publication(models.Model):
    """
    Model representing a publication.
    """

    class Meta:
        ordering = ['-year', '-month', '-id']
        app_label = 'publications_bootstrap'  # Fix for Django<1.7

    # names shown in admin area
    class EMonths(EOrderedChoice):
        JAN = (1, _('January'), 'Jan')
        FEB = (2, _('February'), 'Feb')
        MAR = (3, _('March'), 'Mar')
        APR = (4, _('April'), 'Apr')
        MAY = (5, _('May'), 'May')
        JNE = (6, _('June'), 'Jun')
        JLY = (7, _('July'), 'Jul')
        AUG = (8, _('August'), 'Aug')
        SEP = (9, _('September'), 'Sep')
        OCT = (10, _('October'), 'Oct')
        NOV = (11, _('November'), 'Nov')
        DEC = (12, _('December'), 'Dec')

        def __init__(self, v_, l_, bibtex):
            # abbreviations used in BibTex
            self.bibtex = bibtex

    # Status of the publication
    class EStatuses(EChoice):
        DRAFT = ('d', _('draft'))
        SUBMITTED = ('s', _('submitted'))
        ACCEPTED = ('a', _('accepted'))
        PUBLISHED = ('p', _('published'))

    type = models.ForeignKey(Type)
    citekey = NullCharField(max_length=512, blank=True, null=True, unique=True,
                            help_text='BibTex citation key. Leave blank if unsure.')
    title = models.CharField(max_length=512)
    authors = models.CharField(max_length=2048,
                               help_text='List of authors separated by commas or <i>and</i>.')
    year = models.PositiveIntegerField()
    month = make_echoicefield(EMonths, blank=True, null=True)
    journal = models.CharField(max_length=256, blank=True)
    book_title = models.CharField(max_length=256, blank=True,
                                  help_text='Title of a book, part of which is being cited. See '
                                            'the LATEX book for how to type titles. For book '
                                            'entries, use the `title` field instead')
    publisher = models.CharField(max_length=256, blank=True)
    editor = models.CharField(max_length=256, blank=True,
                              help_text='Name(s) of editor(s), typed as indicated in the LATEX '
                                        'book. If there is also an `author` field, then the '
                                        '`editor` field gives the editor of the book or '
                                        'collection in which the reference appears.')
    edition = models.CharField(max_length=256, blank=True,
                               help_text='The edition of a book -- for example, "Second". This '
                                         'should be an ordinal, and should have the first letter '
                                         'capitalized.')
    institution = models.CharField(max_length=256, blank=True)
    school = models.CharField(max_length=256, blank=True,
                              help_text='The name of the school where a thesis was written.')
    organization = models.CharField(max_length=256, blank=True,
                                    help_text='The organization that sponsors a conference or '
                                              'that publishes a manual.')
    location = models.CharField(max_length=256, blank=True,
                                help_text="Place of publication, location of conference, "
                                          "or publisher's address.")
    country = CountryField(blank=True)
    series = models.CharField(blank=True, max_length=256,
                              help_text='The name of a series or set of books. When citing an '
                                        'entire book, the `title` field gives its title and an '
                                        'optional `series` field gives the name of a series or '
                                        'multi-volume set in which the book is published.')
    volume = models.CharField(blank=True, null=True, max_length=128)
    number = models.CharField(blank=True, null=True, max_length=128, verbose_name='Issue number',
                              help_text='The number of a journal, magazine, technical report, or '
                                        'of a work in a series. An issue of a journal or magazine '
                                        'is usually identified by its volume and number; the '
                                        'organization that issues a technical report usually '
                                        'gives it a number; and sometimes books are given numbers '
                                        'in a named series.')
    chapter = models.CharField(blank=True, null=True, max_length=128)
    section = models.CharField(blank=True, null=True, max_length=128)  # Not officially
    # recognized as Bibtex Field Type
    pages = PagesField(max_length=32, blank=True)
    note = models.CharField(max_length=256, blank=True,
                            help_text='Any additional information that can help the reader. The '
                                      'first word should be capitalized.')
    tags = models.CharField(max_length=256, blank=True,
                            help_text='List of tags separated by commas.')
    url = models.URLField(blank=True, verbose_name='URL', help_text='Link to PDF or journal page.')
    code = models.URLField(blank=True, help_text='Link to page with code.')
    pdf = models.FileField(upload_to='publications_bootstrap/', verbose_name='PDF', blank=True, null=True)
    image = models.ImageField(upload_to='publications_bootstrap/images/', blank=True, null=True)
    thumbnail = models.ImageField(upload_to='publications_bootstrap/thumbnails/', blank=True, null=True)
    external = models.BooleanField(default=False,
                                   help_text='If publication was written in another lab, mark as external.')
    abstract = models.TextField(blank=True)
    doi = NullCharField(max_length=128, verbose_name='DOI', blank=True, null=True, unique=True)
    isbn = NullCharField(max_length=32, verbose_name='ISBN', blank=True, null=True, unique=True,
                         help_text='Only for a book.')  # A-B-C-D
    catalogs = models.ManyToManyField(Catalog, blank=True)
    status = make_echoicefield(EStatuses, default=EStatuses.PUBLISHED, blank=False)

    def __init__(self, *args, **kwargs):
        models.Model.__init__(self, *args, **kwargs)

        # post-process tags
        self.tags = self.tags.replace(';', ',')
        self.tags = self.tags.replace(', and ', ', ')
        self.tags = self.tags.replace(',and ', ', ')
        self.tags = self.tags.replace(' and ', ', ')
        self.tags = [s.strip().lower() for s in self.tags.split(',')]
        self.tags = ', '.join(self.tags).lower()

        self._produce_author_lists()

    def _produce_author_lists(self):
        """
        Parse authors string to create lists of authors.
        """

        # post-process author names
        self.authors = self.authors.replace(', and ', ', ')
        self.authors = self.authors.replace(',and ', ', ')
        self.authors = self.authors.replace(' and ', ', ')
        self.authors = self.authors.replace(';', ',')

        # list of authors
        self.authors_list = [author.strip() for author in self.authors.split(',')]

        # simplified representation of author names
        self.authors_list_simple = []

        # author names represented as a tuple of given and family name
        self.authors_list_split = []

        # tests if title already ends with a punctuation mark
        self.title_ends_with_punct = self.title[-1] in ['.', '!', '?'] if len(self.title) > 0 else False

        suffixes = ['I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', "Jr.", "Sr."]
        prefixes = ['Dr.']
        prepositions = ['van', 'von', 'der', 'de', 'den']

        # further post-process author names
        for i, author in enumerate(self.authors_list):
            if author == '':
                continue

            names = author.split(' ')

            # check if last string contains initials
            if (len(names[-1]) <= 3) \
                    and names[-1] not in suffixes \
                    and all(c in ascii_uppercase for c in names[-1]):
                # turn "Gauss CF" into "C. F. Gauss"
                names = [c + '.' for c in names[-1]] + names[:-1]

            # number of suffixes
            num_suffixes = 0
            for name in names[::-1]:
                if name in suffixes:
                    num_suffixes += 1
                else:
                    break

            # abbreviate names
            for j, name in enumerate(names[:-1 - num_suffixes]):
                # don't try to abbreviate these
                if j == 0 and name in prefixes:
                    continue
                if j > 0 and name in prepositions:
                    continue

                if (len(name) > 2) or (len(name) and (name[-1] != '.')):
                    k = name.find('-')
                    if 0 < k + 1 < len(name):
                        # take care of dash
                        names[j] = name[0] + '.-' + name[k + 1] + '.'
                    else:
                        names[j] = name[0] + '.'

            if len(names):
                self.authors_list[i] = ' '.join(names)

                # create simplified/normalized representation of author name
                if len(names) > 1:
                    for name in names[0].split('-'):
                        name_simple = self.simplify_name(' '.join([name, names[-1]]))
                        self.authors_list_simple.append(name_simple)
                else:
                    self.authors_list_simple.append(self.simplify_name(names[0]))

                # number of prepositions
                num_prepositions = 0
                for name in names:
                    if name in prepositions:
                        num_prepositions += 1

                # splitting point
                sp = 1 + num_suffixes + num_prepositions
                self.authors_list_split.append((' '.join(names[:-sp]), ' '.join(names[-sp:])))

        # list of authors in BibTex format
        self.authors_bibtex = ' and '.join(self.authors_list)

        # overwrite authors string
        if len(self.authors_list) > 2:
            self.authors = ', and '.join([', '.join(self.authors_list[:-1]),
                                          self.authors_list[-1]])
        elif len(self.authors_list) > 1:
            self.authors = ' and '.join(self.authors_list)
        else:
            self.authors = self.authors_list[0]

    def __unicode__(self):
        return self.__str__()

    def __str__(self):
        if len(self.title) < 64:
            return self.title
        else:
            index = self.title.rfind(' ', 40, 62)

            if index < 0:
                return self.title[:61] + '...'
            else:
                return self.title[:index] + '...'

    def tags_escaped(self):
        return [(tag.strip(), urlquote_plus(tag.strip())) for tag in self.tags.split(',')]

    def authors_escaped(self):
        return [(author, author.lower().replace(' ', '+')) for author in self.authors_list]

    def key(self):
        # this publication's first author
        author_lastname = self.authors_list[0].split(' ')[-1]

        publications = Publication.objects.filter(year=self.year,
                                                  authors__icontains=author_lastname).order_by('month', 'id')

        # character to append to BibTex key
        char = ord('a')

        # augment character for every publication 'before' this publication
        for publication in publications:
            if publication == self:
                break
            if publication.authors_list[0].split(' ')[-1] == author_lastname:
                char += 1

        return self.authors_list[0].split(' ')[-1] + str(self.year) + chr(char)

    def title_bibtex(self):
        return self.title.replace('%', r'\%')

    def month_bibtex(self):
        return self.month.bibtex

    def month_long(self):
        if self.month:
            return self.month.label
        return ''

    def first_author(self):
        return self.authors_list[0]

    def journal_or_book_title(self):
        if self.journal:
            return self.journal
        else:
            return self.book_title

    def first_page(self):
        return self.pages.split('-')[0]

    def last_page(self):
        return self.pages.split('-')[-1]

    def z3988(self):
        context_obj = ['ctx_ver=Z39.88-2004']

        if 'django.contrib.sites' in settings.INSTALLED_APPS:
            domain = Site.objects.get_current().domain
        else:
            domain = 'example.com'

        rfr_id = domain.split('.')

        if len(rfr_id) > 2:
            rfr_id = rfr_id[-2]
        elif len(rfr_id) > 1:
            rfr_id = rfr_id[0]
        else:
            rfr_id = ''

        if self.book_title and not self.journal:
            context_obj.append('rft_val_fmt=info:ofi/fmt:kev:mtx:book')
            context_obj.append('rfr_id=info:sid/' + domain + ':' + rfr_id)
            context_obj.append('rft_id=info:doi/' + urlquote_plus(self.doi))

            context_obj.append('rft.btitle=' + urlquote_plus(self.title))

            if self.publisher:
                context_obj.append('rft.pub=' + urlquote_plus(self.publisher))

        else:
            context_obj.append('rft_val_fmt=info:ofi/fmt:kev:mtx:journal')
            context_obj.append('rfr_id=info:sid/' + domain + ':' + rfr_id)
            context_obj.append('rft_id=info:doi/' + urlquote_plus(self.doi))
            context_obj.append('rft.atitle=' + urlquote_plus(self.title))

            if self.journal:
                context_obj.append('rft.jtitle=' + urlquote_plus(self.journal))

            if self.volume:
                context_obj.append('rft.volume={0}'.format(self.volume))

            if self.pages:
                context_obj.append('rft.pages=' + urlquote_plus(self.pages))

            if self.number:
                context_obj.append('rft.issue={0}'.format(self.number))

        if self.month:
            context_obj.append(
                'rft.date={0}-{1}-1'.format(self.year, self.month.value))
        else:
            context_obj.append('rft.date={0}'.format(self.year))

        for author in self.authors_list:
            context_obj.append('rft.au=' + urlquote_plus(author))

        if self.isbn:
            context_obj.append('rft.isbn=' + urlquote_plus(self.isbn))

        return '&'.join(context_obj)

    def clean(self):
        if not self.citekey:
            self._produce_author_lists()
            self.citekey = self.key()

    @staticmethod
    def simplify_name(name):
        name = name.lower()
        name = name.replace(u'ä', u'ae')
        name = name.replace(u'ö', u'oe')
        name = name.replace(u'ü', u'ue')
        name = name.replace(u'ß', u'ss')
        return name



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from django.utils import six

# from . import six

# special character mapping
special_chars = (
    (r'\"{a}', 'ä'), (r'{\"a}', 'ä'), (r'\"a', 'ä'), (r'H{a}', 'ä'),
    (r'\"{A}', 'Ä'), (r'{\"A}', 'Ä'), (r'\"A', 'Ä'), (r'H{A}', 'Ä'),
    (r'\"{o}', 'ö'), (r'{\"o}', 'ö'), (r'\"o', 'ö'), (r'H{o}', 'ö'),
    (r'\"{O}', 'Ö'), (r'{\"O}', 'Ö'), (r'\"O', 'Ö'), (r'H{O}', 'Ö'),
    (r'\"{u}', 'ü'), (r'{\"u}', 'ü'), (r'\"u', 'ü'), (r'H{u}', 'ü'),
    (r'\"{U}', 'Ü'), (r'{\"U}', 'Ü'), (r'\"U', 'Ü'), (r'H{U}', 'Ü'),
    (r'{‘a}', 'à'), (r'\‘A', 'À'),
    (r'{‘e}', 'è'), (r'\‘E', 'È'),
    (r'{‘o}', 'ò'), (r'\‘O', 'Ò'),
    (r'{‘u}', 'ù'), (r'\‘U', 'Ù'),
    (r'{’a}', 'á'), (r'\’A', 'Á'),
    (r'{’e}', 'é'), (r'\’E', 'É'),
    (r'{’o}', 'ó'), (r'\’O', 'Ó'),
    (r'{’u}', 'ú'), (r'\’U', 'Ú'),
    (r'\`a', 'à'), (r'\`A', 'À'),
    (r'\`e', 'è'), (r'\`E', 'È'),
    (r'\`u', 'ù'), (r'\`U', 'Ù'),
    (r'\`o', 'ò'), (r'\`O', 'Ò'),
    (r'\^o', 'ô'), (r'\^O', 'Ô'),
    (r'\^a', 'â'), (r'\^A', 'Â'),
    (r'\^e', 'ê'), (r'\^E', 'Ê'),
    (r'\^i', 'î'), (r'\^I', 'Î'),
    (r'\^u', 'û'), (r'\^U', 'Û'),
    (r'\ss', 'ß'),
    (r'\%', '%'),
    (r'\ae', 'æ'), (r'\AE', 'Æ'), (r'\aa', 'å'), (r'\AA', 'Å'), (r'\o', 'ø'), (r'\O', 'Ø'),
    # swedish, danish, norwegian
    (r'\c{c}', 'ç'), (r'\c{C}', 'Ç'),  # french, portuguese, etc
    (r'\~a', 'ã'), (r'\~A', 'Ã'),  # portuguese
    (r'\~o', 'õ'), (r'\~O', 'Õ'),
    (r'{\i}', 'ı'), (r'\.{I}', 'İ'), ('\\u{g}', 'ğ'), ('\\u{G}', 'Ğ'), (r'\c{s}', 'ş'),
    (r'\c{S}', 'Ş'))  # turkish


def parse(string):
    """
    Takes a string in BibTex format and returns a list of BibTex entries, where
    each entry is a dictionary containing the entries' key-value pairs.

    @type  string: string
    @param string: bibliography in BibTex format

    @rtype: list
    @return: a list of dictionaries representing a bibliography
    """

    # bibliography
    bib = []

    # make sure we are dealing with unicode strings
    if not isinstance(string, six.text_type):
        string = string.decode('utf-8')

    # replace special characters
    for key, value in special_chars:
        string = string.replace(key, value)
    string = re.sub(r'\\[cuHvs]{?([a-zA-Z])}?', r'\1', string)

    # split into BibTex entries
    entries = re.findall(
        r'(?u)@(\w+)[ \t]?{[ \t]*([^,\s]*)[ \t]*,?\s*((?:[^=,\s]+\s*\=\s*(?:"[^"]*"|{(?:[^{}]*|{[^{}]*})*}|[^,}]*),?\s*?)+)\s*}',
        string)

    for entry in entries:
        # parse entry
        pairs = re.findall(r'(?u)([^=,\s]+)\s*\=\s*("[^"]*"|{(?:[^{}]*|{[^{}]*})*}|[^,]*)', entry[2])

        # add to bibliography
        bib.append({'type': entry[0].lower(), 'key': entry[1]})

        for key, value in pairs:
            # post-process key and value
            key = key.lower()
            if value and value[0] == '"' and value[-1] == '"':
                value = value[1:-1]
            if value and value[0] == '{' and value[-1] == '}':
                value = value[1:-1]
            if key not in ['booktitle', 'title']:
                value = value.replace('}', '').replace('{', '')
            else:
                if value.startswith('{') and value.endswith('}'):
                    value = value[1:]
                    value = value[:-1]
            value = value.strip()
            value = re.sub(r'\s+', ' ', value)

            # store pair in bibliography
            bib[-1][key] = value

    return bib



# -*- coding: utf-8 -*-

from django.shortcuts import render

from ..models import Publication


def by_id(request, publication_id):
    try:
        publication = Publication.objects.get(pk=publication_id)

        if 'plain' in request.GET:
            return render(request, 'publications_bootstrap/export/publications.txt', {'publications': [publication]},
                          content_type='text/plain; charset=UTF-8')

        if 'bibtex' in request.GET:
            return render(request, 'publications_bootstrap/export/publications.bib', {'publications': [publication]},
                          content_type='text/x-bibtex; charset=UTF-8')

        if 'mods' in request.GET:
            return render(request, 'publications_bootstrap/export/publications.mods', {'publications': [publication]},
                          content_type='application/xml; charset=UTF-8')

        if 'ris' in request.GET:
            return render(request, 'publications_bootstrap/export/publications.ris', {'publications': [publication]},
                          content_type='application/x-research-info-systems; charset=UTF-8')

        publication.links = publication.publicationlink_set.all()
        publication.files = publication.publicationfile_set.all()

        return render(request, 'publications_bootstrap/pages/id.html', {'publication': publication,
                                                                        'title': publication.type})
    except Publication.DoesNotExist:
        return render(request, 'publications_bootstrap/base.html', {
            'error': True,
            'alert': {'message': "There is no publication with this id: {}".format(publication_id)}},
                      status=404)



__license__ = 'MIT License <http://www.opensource.org/licenses/mit-license.php>'
__author__ = 'Lucas Theis <lucas@theis.io>'
__docformat__ = 'epytext'

try:
    from django.conf.urls import include, url
except ImportError:
    from django.conf.urls.defaults import include, url
from django.contrib import admin

admin.autodiscover()

urlpatterns = [
    url(r'^publications/', include('publications_bootstrap.urls', namespace='publications_bootstrap')),
    url(r'^admin/', include(admin.site.urls)),
]



# -*- coding: utf-8 -*-

import re

from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django_countries import countries

from ..bibtex import parse
from ..models import Publication, Type

# mapping of months
MONTHS = {
    'jan': 1, 'january': 1,
    'feb': 2, 'february': 2,
    'mar': 3, 'march': 3,
    'apr': 4, 'april': 4,
    'may': 5,
    'jun': 6, 'june': 6,
    'jul': 7, 'july': 7,
    'aug': 8, 'august': 8,
    'sep': 9, 'september': 9,
    'oct': 10, 'october': 10,
    'nov': 11, 'november': 11,
    'dec': 12, 'december': 12}

COUNTRIES_BY_CODE = dict(countries)
# Reversed dict
try:
    # Python 2.7.x
    COUNTRIES_BY_NAME = {v: k for k, v in COUNTRIES_BY_CODE.iteritems()}
except:
    # Python 3+
    COUNTRIES_BY_NAME = {v: k for k, v in COUNTRIES_BY_CODE.items()}


def import_bibtex(request):
    if request.method == 'POST':
        # try to parse BibTex
        bib = parse(request.POST['bibliography'])

        # container for error messages
        errors = {}

        # publication types
        types = Type.objects.all()

        # check for errors
        if not bib:
            if not request.POST['bibliography']:
                errors['bibliography'] = 'This field is required.'

        if not errors:
            publications = []

            # try adding publications
            for entry in bib:
                if 'title' in entry and 'author' in entry and 'year' in entry:
                    # parse authors
                    authors = entry['author'].split(' and ')
                    for i in range(len(authors)):
                        author = authors[i].split(',')
                        author = [author[-1]] + author[:-1]
                        authors[i] = ' '.join(author)
                    authors = ', '.join(authors)

                    # add missing keys
                    keys = [
                        'journal',
                        'booktitle',
                        'address',
                        'publisher',
                        'editor',
                        'edition',
                        'institution',
                        'school',
                        'organization',
                        'series',
                        'url',
                        'doi',
                        'isbn',
                        'tags',
                        'note',
                        'abstract',
                        'month']

                    for key in keys:
                        if key not in entry:
                            entry[key] = ''

                    # map integer fields to integers
                    entry['month'] = Publication.EMonths.get(MONTHS.get(entry['month'].lower(), 0), None)

                    for field in ['volume', 'number', 'chapter', 'section']:
                        entry[field] = entry.get(field, None)

                    # remove whitespace characters (likely due to line breaks)
                    entry['url'] = re.sub(r'\s', '', entry['url'])

                    if 'country' not in entry:
                        entry['country'] = ''
                    else:
                        if entry['country'].strip() in COUNTRIES_BY_NAME:
                            entry['country'] = COUNTRIES_BY_NAME[entry['country'].strip()]
                        elif entry['country'].upper() in COUNTRIES_BY_CODE:
                            entry['country'] = entry['country'].upper()
                        else:
                            entry['country'] = ''

                    # determine type
                    type_id = None

                    for t in types:
                        if entry['type'] in t.bibtex_type_list:
                            type_id = t.id
                            break

                    if type_id is None:
                        errors['bibliography'] = 'Type "{}" unknown.'.format(entry['type'])
                        break

                    # add publication
                    publications.append(Publication(
                        type_id=type_id,
                        citekey=entry['key'],
                        title=entry['title'],
                        authors=authors,
                        year=entry['year'],
                        month=entry['month'],
                        journal=entry['journal'],
                        book_title=entry['booktitle'],
                        publisher=entry['publisher'],
                        location=entry['address'],
                        country=entry['country'],
                        editor=entry['editor'],
                        edition=entry['edition'],
                        institution=entry['institution'],
                        school=entry['school'],
                        organization=entry['organization'],
                        series=entry['series'],
                        volume=entry['volume'],
                        number=entry['number'],
                        chapter=entry['chapter'],
                        section=entry['section'],
                        note=entry['note'],
                        url=entry['url'],
                        doi=entry['doi'],
                        isbn=entry['isbn'],
                        external=False,
                        abstract=entry['abstract'],
                        tags=entry['tags'],
                        status=Publication.EStatuses.PUBLISHED))
                else:
                    errors['bibliography'] = 'Make sure that the keys <title>, <author> and <year> are present.'
                    break

            if not publications:
                errors['bibliography'] = 'No valid BibTex entries found.'

        if errors:
            # some error occurred
            return render(
                request,
                'admin/publications_bootstrap/import_bibtex.html', {
                    'errors': errors,
                    'title': 'Import BibTex',
                    'types': Type.objects.all(),
                    'request': request})
        else:
            try:
                # save publications
                for publication in publications:
                    publication.save()
            except:
                msg = 'Some error occurred during saving of publications.'
            else:
                if len(publications) > 1:
                    msg = 'Successfully added {} publications.'.format(len(publications))
                else:
                    msg = 'Successfully added {} publication.'.format(len(publications))

            # show message
            messages.info(request, msg)

            # redirect to publication listing
            return HttpResponseRedirect('../')
    else:
        return render(request, 'admin/publications_bootstrap/import_bibtex.html', {'title': 'Import BibTex',
                                                                                   'types': Type.objects.all(),
                                                                                   'request': request})


import_bibtex = staff_member_required(import_bibtex)



from django.apps import AppConfig


class PublicationsBootstrapConfig(AppConfig):
    name = 'publications_bootstrap'
    verbose_name = "Bootstrap-powered scientific publications for Django"

    # TODO: check if dependencies are met



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__license__ = 'MIT License <http://www.opensource.org/licenses/mit-license.php>'
__authors__ = ['Lucas Theis <lucas@theis.io>', 'Marc Bourqui <https://github.com/mbourqui>']
__docformat__ = 'epytext'
__version__ = '2.0.0'
__version_info__ = tuple([int(num) if num.isdigit() else num for num in __version__.replace('-', '.', 1).split('.')])
__status__ = 'Stable'



# -*- coding: utf-8 -*-

from .models import PublicationLink, PublicationFile


def populate(publications):
    """
    Load publication links and files from database and attach to publications
    """

    publication_links = PublicationLink.objects.filter(publication__in=publications)
    publication_files = PublicationFile.objects.filter(publication__in=publications)

    publications_ = {}
    for publication in publications:
        publication.links = []
        publication.files = []
        publications_[publication.id] = publication

    for link in publication_links:
        publications_[link.publication_id].links.append(link)
    for file in publication_files:
        publications_[file.publication_id].files.append(file)



# -*- coding: utf-8 -*-

from distutils.version import StrictVersion

import django
from django import forms
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

try:
    from django.conf.urls import url
except ImportError:
    # Fix for Django<1.6
    from django.conf.urls.defaults import url

from .. import admin_views
from ..models import Publication, PublicationLink, PublicationFile


class PublicationLinkInline(admin.StackedInline):
    model = PublicationLink
    extra = 1
    max_num = 5


class PublicationFileInline(admin.StackedInline):
    model = PublicationFile
    extra = 1
    max_num = 5


class PublicationAdminForm(forms.ModelForm):
    class Meta:
        model = Publication
        # Fix for Django<1.7
        if StrictVersion(django.get_version()) >= StrictVersion('1.7.0'):
            fields = '__all__'

    def __init__(self, *args, **kwargs):
        super(PublicationAdminForm, self).__init__(*args, **kwargs)

        # give (usually) numeric fields appearance of IntegerField
        for field in ['volume', 'number', 'chapter', 'section']:
            self.fields[field].widget.attrs['class'] = 'vIntegerField'


class PublicationAdmin(admin.ModelAdmin):
    form = PublicationAdminForm
    list_display = ('type', 'first_author', 'title', 'type', 'year', 'journal_or_book_title', 'status',)
    list_display_links = ('title',)
    list_filter = ('year', 'journal', 'status', 'catalogs',)
    change_list_template = 'admin/publications_bootstrap/publication_change_list.html'
    search_fields = (
        'title', 'journal', 'book_title', 'authors', 'tags', 'year', 'institution', 'school', 'organization')
    actions = ['set_status_draft', 'set_status_submitted', 'set_status_accepted', 'set_status_published']
    fieldsets = (
        (None, {
            'fields': ('type', 'title', 'authors', 'year', 'month', 'status', 'external')}),
        ('All available fields (overridden if set below)', {
            'classes': ('collapse',),
            'fields': (
                'book_title', 'publisher', 'editor', 'edition', 'institution', 'school', 'organization', 'location',
                'country', 'volume', 'number', 'series', 'chapter', 'section', 'pages', 'url')}),
        ('Journal', {
            'classes': ('collapse',),
            'fields': ('journal', 'volume', 'number', 'pages')}),
        ('Conference', {
            'classes': ('collapse',),
            'fields': (
                'book_title', 'editor', 'volume', 'number', 'series', 'pages', 'location', 'country', 'organization',
                'publisher')}),
        ('Technical Report', {
            'classes': ('collapse',),
            'fields': ('institution', 'number', 'location', 'country')}),
        ('Book or Manual', {
            'classes': ('collapse',),
            'fields': (
                'editor', 'publisher', 'volume', 'number', 'series', 'organization', 'location', 'country',
                'edition')}),
        ('In Book or Collection', {
            'classes': ('collapse',),
            'fields': ('book_title', 'editor', 'chapter', 'pages', 'publisher', 'volume', 'number',
                       'series', 'location', 'country', 'edition')}),
        ('Thesis', {
            'classes': ('collapse',),
            'fields': ('school', 'location', 'country')}),
        ('References', {
            'fields': ('citekey', 'tags', 'code', 'url', 'doi', 'isbn')}),
        ('Description', {
            'fields': ('abstract', 'note')}),
        ('Media', {
            'classes': ('collapse',),
            'fields': ('pdf', 'image', 'thumbnail')}),
        ('Catalogs', {
            'fields': ('catalogs',)}),
    )
    inlines = [PublicationLinkInline, PublicationFileInline]

    def get_urls(self):
        return [url(r'^import_bibtex/$', admin_views.import_bibtex, name='publications_publication_import_bibtex'),
                ] + super(PublicationAdmin, self).get_urls()

    def _set_status(self, request, queryset, new_status):
        rows_updated = queryset.update(status=new_status)
        if rows_updated == 1:
            message_bit = "1 publication was"
        else:
            message_bit = "{:d} publications were".format(rows_updated)
        self.message_user(request, "{} successfully marked as {}.".format(message_bit, new_status.label))

    def set_status_draft(self, request, queryset):
        self._set_status(request, queryset, Publication.EStatuses.DRAFT)

    set_status_draft.short_description = _("Mark selected %(verbose_name_plural)s as drafts")

    def set_status_submitted(self, request, queryset):
        self._set_status(request, queryset, Publication.EStatuses.SUBMITTED)

    set_status_submitted.short_description = _("Mark selected %(verbose_name_plural)s as submitted")

    def set_status_accepted(self, request, queryset):
        self._set_status(request, queryset, Publication.EStatuses.ACCEPTED)

    set_status_accepted.short_description = _("Mark selected %(verbose_name_plural)s as accepted")

    def set_status_published(self, request, queryset):
        self._set_status(request, queryset, Publication.EStatuses.PUBLISHED)

    set_status_published.short_description = _("Mark selected %(verbose_name_plural)s as published")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import django
from django.test.runner import DiscoverRunner

os.environ['DJANGO_SETTINGS_MODULE'] = 'publications_bootstrap.tests.settings'

if __name__ == "__main__":
    # https://docs.djangoproject.com/en/1.8/topics/settings/#calling-django-setup-is-required-for-standalone-django-usage

    django.setup()

    sys.exit(DiscoverRunner(verbosity=1).run_tests(['publications_bootstrap']))



# -*- coding: utf-8 -*-

from time import sleep

from django.contrib.auth.models import User
from django.test import LiveServerTestCase
from selenium import webdriver

from ..models import Publication
from ..tests import tests


class LiveTests(LiveServerTestCase):
    fixtures = ['initial_data.json', 'test_data.json']
    urls = 'publications_bootstrap.tests.urls'

    @classmethod
    def setUpClass(cls):
        cls.selenium = webdriver.PhantomJS()
        super(LiveTests, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super(LiveTests, cls).tearDownClass()

    def setUp(self):
        User.objects.create_superuser('admin', 'admin@test.de', 'admin')

        # give the browser a little time
        sleep(1.)

        # login
        self.selenium.get('{0}{1}'.format(self.live_server_url, '/admin/'))
        username_input = self.selenium.find_element_by_name("username")
        username_input.send_keys('admin')
        password_input = self.selenium.find_element_by_name("password")
        password_input.send_keys('admin')
        self.selenium.find_element_by_xpath('//input[@value="Log in"]').click()

    def test_import_bibtex(self):
        count = Publication.objects.count()

        self.selenium.get('{0}{1}'.format(
            self.live_server_url,
            '/admin/publications_bootstrap/publication/import_bibtex/'))
        bibliography_input = self.selenium.find_element_by_name("bibliography")
        bibliography_input.send_keys(tests.TEST_BIBLIOGRAPHY)
        self.selenium.find_element_by_xpath('//input[@value="Import"]').click()

        self.assertEqual(Publication.objects.count() - count, tests.TEST_BIBLIOGRAPHY_COUNT)

    def test_import_bibtex_button(self):
        count = Publication.objects.count()

        self.selenium.get(
            '{0}{1}'.format(self.live_server_url, '/admin/publications_bootstrap/publication/'))
        self.selenium.find_element_by_link_text('Import BibTex').click()
        self.selenium.find_element_by_xpath('//input[@value="Import"]').click()



# -*- coding: utf-8 -*-
from distutils.version import StrictVersion

import django
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.http import HttpRequest
from django.template import Template, RequestContext
from django.test import TestCase

from ..models import Publication, Type, PublicationLink, Catalog
from ..templatetags.publication_extras import tex_parse


class Tests(TestCase):
    fixtures = ['initial_data.json', 'test_data.json']
    urls = 'publications_bootstrap.tests.urls'

    def setUp(self):
        User.objects.create_superuser('admin', 'admin@test.de', 'admin')

    def test_authors(self):
        publication = Publication.objects.create(
            type=Type.objects.get(pk=1),
            authors=u'Jörn-Philipp Lies and Ralf M. Häfner and M. Bethge',
            title=u'Slowness and sparseness have diverging effects on complex cell learning',
            year=2014,
            journal=u'PLoS Computational Biology',
            external=0)
        publication.clean()
        publication.save()

        self.assertEqual(len(publication.authors_list), 3)
        self.assertTrue('J.-P. Lies' in publication.authors_list)
        self.assertTrue(('J.-P.', 'Lies') in publication.authors_list_split)

    def test_citekey(self):
        publication = Publication.objects.create(
            type=Type.objects.get(pk=1),
            authors=u'A. Unique and B. Common',
            title=u'Title 1',
            year=2014,
            journal=u'Journal',
            external=0)
        publication.clean()
        publication.save()

        self.assertEqual(publication.citekey, 'Unique2014a')

        publication = Publication.objects.create(
            type=Type.objects.get(pk=1),
            authors=u'A. Unique and C. Common',
            title=u'Title 2',
            year=2014,
            journal=u'Journal',
            external=0)
        publication.clean()
        publication.save()

        self.assertEqual(publication.citekey, 'Unique2014b')

        publication = Publication.objects.create(
            type=Type.objects.get(pk=1),
            authors=u'A. Unique and D. Uncommon',
            title=u'Title 3',
            year=2013,
            journal=u'Journal',
            external=0)
        publication.clean()
        publication.save()

        self.assertEqual(publication.citekey, 'Unique2013a')

    def test_publication_links(self):
        link = PublicationLink.objects.create(publication_id=1, description='Test', url='http://test.com')
        link.save()

        self.assertEqual(self.client.get('/publications/').status_code, 200)
        self.assertEqual(self.client.get('/publications/1/').status_code, 200)

    def test_publications(self):
        publication = Publication.objects.create(
            type=Type.objects.get(pk=1),
            authors=u'Jörn-Philipp Lies and Ralf M. Häfner and M. Bethge',
            title=u'Slowness and sparseness have diverging effects on complex cell learning',
            year=2014,
            journal=u'PLoS Computational Biology',
            external=0)
        publication.clean()
        publication.save()

        self.assertEqual(self.client.get('/publications/').status_code, 200)
        self.assertEqual(self.client.get('/publications/?plain').status_code, 200)
        self.assertEqual(self.client.get('/publications/?bibtex').status_code, 200)
        self.assertEqual(self.client.get('/publications/?mods').status_code, 200)
        self.assertEqual(self.client.get('/publications/?ris').status_code, 200)
        self.assertEqual(self.client.get('/publications/?rss').status_code, 200)
        self.assertEqual(self.client.get('/publications/1/').status_code, 200)
        self.assertEqual(self.client.get('/publications/1/?plain').status_code, 200)
        self.assertEqual(self.client.get('/publications/1/?bibtex').status_code, 200)
        self.assertEqual(self.client.get('/publications/1/?mods').status_code, 200)
        self.assertEqual(self.client.get('/publications/1/?ris').status_code, 200)
        self.assertEqual(self.client.get('/publications/100/').status_code, 404)
        response = self.client.get('/publications/j.-p.+lies/')
        self.assertEqual(response.status_code, 200)
        self.assertGreater(len(response.context['publications']), 0)
        self.assertEqual(self.client.get('/publications/j.-p.+lies/?plain').status_code, 200)
        self.assertEqual(self.client.get('/publications/j.-p.+lies/?bibtex').status_code, 200)
        self.assertEqual(self.client.get('/publications/j.-p.+lies/?mods').status_code, 200)
        self.assertEqual(self.client.get('/publications/j.-p.+lies/?ris').status_code, 200)
        self.assertEqual(self.client.get('/publications/j.-p.+lies/?rss').status_code, 200)
        self.assertEqual(self.client.get('/publications/tag/noise+correlations/').status_code, 200)
        self.assertEqual(self.client.get('/publications/tag/noise+correlations/?plain').status_code, 200)
        self.assertEqual(self.client.get('/publications/tag/noise+correlations/?bibtex').status_code, 200)
        self.assertEqual(self.client.get('/publications/tag/noise+correlations/?mods').status_code, 200)
        self.assertEqual(self.client.get('/publications/tag/noise+correlations/?ris').status_code, 200)
        self.assertEqual(self.client.get('/publications/catalog/highlights/').status_code, 200)
        self.assertEqual(self.client.get('/publications/catalog/highlights/?plain').status_code, 200)
        self.assertEqual(self.client.get('/publications/catalog/highlights/?bibtex').status_code, 200)
        self.assertEqual(self.client.get('/publications/catalog/highlights/?mods').status_code, 200)
        self.assertEqual(self.client.get('/publications/catalog/highlights/?ris').status_code, 200)
        self.assertEqual(self.client.get('/publications/catalog/highlights/?rss').status_code, 200)
        self.assertEqual(self.client.get('/publications/catalog/foobar/').status_code, 404)
        self.assertEqual(self.client.get('/publications/year/2011/').status_code, 200)
        self.assertEqual(self.client.get('/publications/year/2011/?plain').status_code, 200)
        self.assertEqual(self.client.get('/publications/year/2011/?bibtex').status_code, 200)

        publication = Publication.objects.create(
            type=Type.objects.get(pk=1),
            authors=u'A. Unique and B. Common',
            title=u'Title 3',
            year=2012,
            journal=u'Journal',
            external=0)
        publication.clean()
        publication.save()

        publication = Publication.objects.create(
            type=Type.objects.get(pk=1),
            authors=u'A. Unique and C. Common and D. Someone',
            title=u'Title 4',
            year=2011,
            journal=u'Journal',
            external=0)
        publication.clean()
        publication.save()

        link = PublicationLink.objects.create(publication_id=publication.id, description='Test', url='http://test.com')
        link.save()

        response = self.client.get('/publications/c.+common/')

        self.assertTrue('C. Common' in str(response.content))
        self.assertFalse('B. Common' in str(response.content))

    def test_bibtex_import(self):
        self.client.login(username='admin', password='admin')

        count = Publication.objects.count()
        self.client.post('/admin/publications_bootstrap/publication/import_bibtex/',
                         {'bibliography': TEST_BIBLIOGRAPHY}, follow=False)

        self.assertEqual(Publication.objects.count() - count, TEST_BIBLIOGRAPHY_COUNT)

        publications = Publication.objects.filter(citekey='test:2009')

        self.assertEqual(len(publications), 1)
        self.assertTrue('F. Last-Name' in publications[0].authors_list)
        self.assertTrue('P. van der Markt III' in publications[0].authors_list)
        self.assertTrue('Test' in publications[0].authors_list)
        self.assertTrue('C. F. Gauss II' in publications[0].authors_list)

        publications = Publication.objects.filter(citekey='kay2015good')

        self.assertEqual(len(publications), 1)
        self.assertTrue(publications[0].title.startswith('How Good is 85%?'))

    def test_unapi(self):
        self.assertEqual(self.client.get('/publications/unapi/').status_code, 200)
        self.assertEqual(self.client.get('/publications/unapi/?id=1').status_code, 200)
        self.assertEqual(self.client.get('/publications/unapi/?id=1&format=mods').status_code, 200)
        self.assertEqual(self.client.get('/publications/unapi/?id=1&format=bibtex').status_code, 200)
        self.assertEqual(self.client.get('/publications/unapi/?id=1&format=ris').status_code, 200)
        self.assertEqual(self.client.get('/publications/unapi/?id=99999&format=bibtex').status_code, 404)
        self.assertEqual(self.client.get('/publications/unapi/?id=1&format=foobar').status_code, 406)

    def test_admin(self):
        self.client.login(username='admin', password='admin')

        self.assertEqual(self.client.get('/publications/').status_code, 200)
        self.assertEqual(self.client.get('/admin/publications_bootstrap/type/6/move-up/', follow=True).status_code, 200)
        self.assertEqual(self.client.get('/admin/publications_bootstrap/type/6/move-down/', follow=True).status_code,
                         200)

        # Test admin actions
        from django.contrib.admin import ACTION_CHECKBOX_NAME
        change_url = reverse('admin:publications_bootstrap_publication_changelist')
        for action, db_value in [('set_status_draft', Publication.EStatuses.DRAFT),
                                 ('set_status_submitted', Publication.EStatuses.SUBMITTED),
                                 ('set_status_accepted', Publication.EStatuses.ACCEPTED),
                                 ('set_status_published', Publication.EStatuses.PUBLISHED), ]:
            data = {'action': action,
                    ACTION_CHECKBOX_NAME: Publication.objects.all().values_list('pk', flat=True)}
            response = self.client.post(change_url, data, follow=True)

            # Test effective change in DB
            measured = Publication.objects.filter(status=db_value, pk__in=data[ACTION_CHECKBOX_NAME]).count()
            expected = Publication.objects.count()
            self.assertEqual(measured, expected, "AssertionError in {}: {} != {}".format(action, measured, expected, ))
            # Test UI
            # For some reason, the <ul class="messagelist"> is not shown on D1.10
            if StrictVersion(django.get_version()) < StrictVersion('1.10.0'):
                self.assertContains(response, '{} publications were successfully marked as '
                                              ''.format(expected),
                                    msg_prefix="AssertionError in {}: ".format(action))
                # Test on a single object
                # Django<1.6 does not support QuerySet.first()
                data[ACTION_CHECKBOX_NAME] = [Publication.objects.all()[0].pk]
                response = self.client.post(change_url, data, follow=True)
                self.assertContains(response, '1 publication was successfully marked as ',
                                    msg_prefix="AssertionError in {}: ".format(action))

    def test_extras(self):
        link = PublicationLink.objects.create(publication_id=1, description='Test', url='http://test.com')
        link.save()

        publication = Publication.objects.get(pk=1)
        highlights_catalog = Catalog.objects.get(title__iexact='highlights')

        # add publication to catalog
        publication.catalogs.add(highlights_catalog)

        # Create empty Catalog
        naughty_catalog = Catalog(title="Naughty")
        naughty_catalog.save()

        # render catalog
        tpl = Template("""
{% load publication_extras %}
{% get_publication 1 %}
{% get_catalog 'highlights' 'publications_bootstrap/components/publications_with_thumbnails.html' %}
{% get_catalog 'naughty' %}
{% get_catalog 'foobar' %}
{% get_publication 100 %}
{% get_publications %}
""")

        self.assertGreater(len(tpl.render(RequestContext(HttpRequest())).strip()), 0)

        # tex_parse is used to replace simple LaTeX code in publication titles
        self.assertEqual(tex_parse(u'$L_p$-spherical'), u'L<sub>p</sub>-spherical')
        self.assertEqual(tex_parse(u'$L^2$-spherical'), u'L<sup>2</sup>-spherical')


TEST_BIBLIOGRAPHY_COUNT = 10
TEST_BIBLIOGRAPHY = r"""
@article{Bethge2002c,
  author = "M. Bethge and D. Rotermund and K. Pawelzik",
  title = "Optimal short-term population coding: when Fisher information fails",
  year = 2002,
  journal = "Neural Computation",
  month = "Oct",
  keywords = "population coding, fisher information",
  doi = "10.1162/08997660260293247",
  url = "http://www.mitpressjournals.org/doi/abs/10.1162/08997660260293247"
}

@article{Simovski2011,
  author =        {Simovski, Constantin R.},
  journal =       {J. Opt.},
  month =         jan,
  number =        {1},
  pages =         {013001},
  title =         {{On electromagnetic characterization and
                   homogenization of nanostructured metamaterials}},
  volume =        {13},
  year =          {2011},
  doi =           {10.1088/2040-8978/13/1/013001},
  issn =          {2040-8978},
  url =           {http://stacks.iop.org/2040-8986/13/i=1/
                  a=013001?key=crossref.7321766a6630b917c6f066f2abc1e2cc},
}

@inproceedings{gerwinn2008bayesian,
  title={Bayesian inference for spiking neuron models with a sparsity prior},
  author={Gerwinn, Sebastian and Macke, Jakob and Seeger, Matthias and Bethge, Matthias},
  booktitle={Proceedings of the 21st Annual Conference on Neural Information Processing Systems},
  number={EPFL-CONF-161311},
  pages={529--536},
  year={2008},
  country={CH}
}

@article{hafner2000dynamical,
  title={A dynamical model of the inner Galaxy},
  author={H{\"a}fner, Ralf and Evans, N Wyn and Dehnen, Walter and Binney, James},
  journal={Monthly Notices of the Royal Astronomical Society},
  volume={314},
  number={3},
  pages={433--452},
  year={2000},
  publisher={Oxford University Press}
}

@misc{test:2009,
    title = "Test",
    author = {Last-Name, First and Peter van der Markt III and Test and Gauss II CF},
    year = 2009,
    country = {Wallis and Futuna},
    pages = {1-2}
}

@article{DBLP:journals/corr/KummererWB14,
 author    = {Matthias K{\"{u}}mmerer and
              Thomas Wallis and
              Matthias Bethge},
 title     = {How close are we to understanding image-based saliency?},
 journal   = {CoRR},
 year      = {2014},
 volume    = {abs/1409.7686},
 url       = {http://arxiv.org/abs/1409.7686},
 timestamp = {Mon, 27 Oct 2014 13:50:21 +0100},
 biburl    = {http://dblp.uni-trier.de/rec/bib/journals/corr/KummererWB14},
 bibsource = {dblp computer science bibliography, http://dblp.org}
}

@ARTICLE{2014arXiv1411.1045K,
   author = {{K{\"u}mmerer}, M. and {Theis}, L. and {Bethge}, M.},
    title = "{Deep Gaze I: Boosting Saliency Prediction with Feature Maps Trained on ImageNet}",
  journal = {ArXiv e-prints},
archivePrefix = "arXiv",
   eprint = {1411.1045},
 primaryClass = "cs.CV",
 keywords = {Computer Science - Computer Vision and Pattern Recognition, Quantitative Biology - Neurons and Cognition, Statistics - Applications},
     year = 2014,
    month = nov,
   adsurl = {http://adsabs.harvard.edu/abs/2014arXiv1411.1045K},
  adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
@incollection{dougan2014objective,
  title={Objective Functions},
  author={Do{\u{g}}an, Haluk and Otu, Hasan H},
  booktitle={Multiple Sequence Alignment Methods},
  pages={45--58},
  year={2014},
  publisher={Springer}
}

@inproceedings{DBLP:conf/patmos/ShahWSB14,
  author    = {Syed Abbas Ali Shah and
               Jan Wagner and
               Thomas Schuster and
               Mladen Berekovic},
  title     = {A lightweight-system-level power and area estimation methodology for
               application specific instruction set processors},
  booktitle = {24th International Workshop on Power and Timing Modeling, Optimization
               and Simulation, PATMOS), Palma de Mallorca, Spain, September 29 -
               Oct. 1, 2014},
  pages     = {1--5},
  publisher = {{IEEE}},
  year      = {2014},
  url       = {http://dx.doi.org/10.1109/PATMOS.2014.6951886},
  doi       = {10.1109/PATMOS.2014.6951886},
  timestamp = {Tue, 18 Nov 2014 12:34:31 +0100},
  biburl    = {http://dblp.uni-trier.de/rec/bib/conf/patmos/ShahWSB14},
  bibsource = {dblp computer science bibliography, http://dblp.org},
  isbn      = {978-1-4799-5412-4}
}


@inproceedings{kay2015good,
  title={How Good is 85\%? A Survey Tool to Connect Classifier Evaluation to Acceptability of Accuracy},
  author={Kay, Matthew and Patel, Shwetak N and Kientz, Julie A},
  booktitle={Proceedings of the 33rd Annual ACM Conference on Human Factors in Computing Systems},
  pages={347--356},
  year={2015},
  organization={ACM},
  address = {New York, NY},
  country = {USA}
}
"""



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core import management
from django.db import migrations

app_label = 'publications_bootstrap'
fixture = 'initial_data'


def load_fixtures(apps, schema_editor):
    management.call_command('loaddata', fixture, app_label=app_label)


class Migration(migrations.Migration):
    dependencies = [
        (app_label, '0001_initial'),
    ]

    operations = [
        migrations.RunPython(load_fixtures),
    ]



# -*- coding: utf-8 -*-

try:
    from django.conf.urls import url
except ImportError:
    from django.conf.urls.defaults import url

from . import views

app_name = 'publications_bootstrap'
urlpatterns = [
    url(r'^$', views.by_year, name='index'),
    url(r'^(?P<publication_id>\d+)/$', views.by_id, name='id'),
    url(r'^year/(?P<year>\d+)/$', views.by_year, name='year'),
    url(r'^tag/(?P<tag>.+)/$', views.by_tag, name='tag'),
    url(r'^catalog/(?P<title>.+)/$', views.for_catalog, name='catalog'),
    url(r'^unapi/$', views.by_unapi, name='unapi'),
    url(r'^(?P<name>.+)/$', views.by_author, name='author'),
]



# -*- coding: utf-8 -*-

from django.contrib import admin

from .listadmin import ListAdmin
from .publicationadmin import PublicationAdmin
from .typeadmin import TypeAdmin
from ..models import Type, Catalog, Publication

admin.site.register(Type, TypeAdmin)
admin.site.register(Catalog, ListAdmin)
admin.site.register(Publication, PublicationAdmin)



import os

from django.conf import global_settings

BASE_DIR = os.path.dirname(__file__)

DEBUG = False

SECRET_KEY = 'm+qa*7_8t-=17zt_)9gi)4g%6w*v$xxkh6rwrys*bn9su+5%du'

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.staticfiles',

    'ordered_model',

    'publications_bootstrap',
)
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}
MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

LANGUAGE_CODE = 'en'

ROOT_URLCONF = 'publications_bootstrap.tests.urls'

MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
MEDIA_URL = '/media/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static')
STATIC_URL = '/static/'

SITE_ID = 1

USE_I18N = True

if hasattr(global_settings, 'TEMPLATE_CONTEXT_PROCESSORS'):
    TEMPLATE_CONTEXT_PROCESSORS = tuple(global_settings.TEMPLATE_CONTEXT_PROCESSORS) + (
        'django.core.context_processors.request',)
else:
    TEMPLATES = [
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': [],
            'APP_DIRS': True,
            'OPTIONS': {
                'context_processors': [
                    'django.template.context_processors.request',
                ],
            },
        },
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from setuptools import setup, find_packages

from publications_bootstrap import __version__

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-publications-bootstrap',
    version=__version__,
    author='Marc Bourqui',
    author_email='https://github.com/mbourqui',
    license='MIT',
    description='A Django app for managing scientific publications with a Bootstrap-powered UI.',
    long_description=README,
    url='https://github.com/mbourqui/django-publications-bootstrap',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Django>=1.9.13',
        'Pillow>=2.4.0',
        'django-countries>=4.0',
        'django-ordered-model>=1.4.1',
        'six>=1.10.0',
        'django-echoices>=2.1.0',
    ],
    zip_safe=False,
    keywords='django scientific publications citations references bibliography',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django_countries.fields
import publications_bootstrap.fields
import echoices.fields
import publications_bootstrap.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PublicationFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=256)),
                ('file', models.FileField(upload_to='publications_bootstrap/')),
            ],
        ),
        migrations.CreateModel(
            name='PublicationLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=256)),
                ('url', models.URLField(verbose_name='URL')),
            ],
        ),
        migrations.CreateModel(
            name='Catalog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=128, unique=True)),
                ('description', models.CharField(max_length=128)),
            ],
            options={
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='Publication',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('citekey', publications_bootstrap.fields.NullCharField(blank=True, help_text='BibTex citation key. Leave blank if unsure.', max_length=512, null=True, unique=True)),
                ('title', models.CharField(max_length=512)),
                ('authors', models.CharField(help_text='List of authors separated by commas or <i>and</i>.', max_length=2048)),
                ('year', models.PositiveIntegerField()),
                ('month', echoices.fields.make_echoicefield(blank=True, echoices=publications_bootstrap.models.Publication.EMonths, null=True),),
                ('journal', models.CharField(blank=True, max_length=256)),
                ('book_title', models.CharField(blank=True, help_text='Title of a book, part of which is being cited. See the LATEX book for how to type titles. For book entries, use the `title` field instead', max_length=256)),
                ('publisher', models.CharField(blank=True, max_length=256)),
                ('editor', models.CharField(blank=True, help_text='Name(s) of editor(s), typed as indicated in the LATEX book. If there is also an `author` field, then the `editor` field gives the editor of the book or collection in which the reference appears.', max_length=256)),
                ('edition', models.CharField(blank=True, help_text='The edition of a book -- for example, "Second". This should be an ordinal, and should have the first letter capitalized.', max_length=256)),
                ('institution', models.CharField(blank=True, max_length=256)),
                ('school', models.CharField(blank=True, help_text='The name of the school where a thesis was written.', max_length=256)),
                ('organization', models.CharField(blank=True, help_text='The organization that sponsors a conference or that publishes a manual.', max_length=256)),
                ('location', models.CharField(blank=True, help_text="Place of publication, location of conference, or publisher's address.", max_length=256)),
                ('country', django_countries.fields.CountryField(blank=True, max_length=2)),
                ('series', models.CharField(blank=True, help_text='The name of a series or set of books. When citing an entire book, the `title` field gives its title and an optional `series` field gives the name of a series or multi-volume set in which the book is published.', max_length=256)),
                ('volume', models.CharField(blank=True, max_length=128, null=True)),
                ('number', models.CharField(blank=True, help_text='The number of a journal, magazine, technical report, or of a work in a series. An issue of a journal or magazine is usually identified by its volume and number; the organization that issues a technical report usually gives it a number; and sometimes books are given numbers in a named series.', max_length=128, null=True, verbose_name='Issue number')),
                ('chapter', models.CharField(blank=True, max_length=128, null=True)),
                ('section', models.CharField(blank=True, max_length=128, null=True)),
                ('pages', publications_bootstrap.fields.PagesField(blank=True, max_length=32)),
                ('note', models.CharField(blank=True, help_text='Any additional information that can help the reader. The first word should be capitalized.', max_length=256)),
                ('tags', models.CharField(blank=True, help_text='List of tags separated by commas.', max_length=256)),
                ('url', models.URLField(blank=True, help_text='Link to PDF or journal page.', verbose_name='URL')),
                ('code', models.URLField(blank=True, help_text='Link to page with code.')),
                ('pdf', models.FileField(blank=True, null=True, upload_to='publications_bootstrap/', verbose_name='PDF')),
                ('image', models.ImageField(blank=True, null=True, upload_to='publications_bootstrap/images/')),
                ('thumbnail', models.ImageField(blank=True, null=True, upload_to='publications_bootstrap/thumbnails/')),
                ('external', models.BooleanField(default=False, help_text='If publication was written in another lab, mark as external.')),
                ('abstract', models.TextField(blank=True)),
                ('doi', publications_bootstrap.fields.NullCharField(blank=True, max_length=128, null=True, unique=True, verbose_name='DOI')),
                ('isbn', publications_bootstrap.fields.NullCharField(blank=True, help_text='Only for a book.', max_length=32, null=True, unique=True, verbose_name='ISBN')),
                ('status', echoices.fields.make_echoicefield(default=publications_bootstrap.models.Publication.EStatuses.PUBLISHED, echoices=publications_bootstrap.models.Publication.EStatuses)),
                ('catalogs', models.ManyToManyField(blank=True, to='publications_bootstrap.Catalog')),
            ],
            options={
                'ordering': ['-year', '-month', '-id'],
            },
        ),
        migrations.CreateModel(
            name='Type',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveIntegerField(db_index=True, editable=False)),
                ('title', models.CharField(max_length=128, unique=True)),
                ('description', models.CharField(max_length=128)),
                ('bibtex_types', models.CharField(default='article', help_text='Possible BibTex types, separated by comma.', max_length=256, verbose_name='BibTex types')),
                ('hidden', models.BooleanField(default=False, help_text='Hide publications from main view.')),
            ],
            options={
                'ordering': ('order',),
            },
        ),
        migrations.AddField(
            model_name='publication',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='publications_bootstrap.Type'),
        ),
        migrations.AddField(
            model_name='publicationlink',
            name='publication',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='publications_bootstrap.Publication'),
        ),
        migrations.AddField(
            model_name='publicationfile',
            name='publication',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='publications_bootstrap.Publication'),
        ),
    ]



# -*- coding: utf-8 -*-

from .type import Type
from .catalog import Catalog
from .publication import Publication
from .publicationlink import PublicationLink
from .publicationfile import PublicationFile
